///////////////////////////////////////////////////////////////////////////////
// PjTomography Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;


///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet.
///////////////////////////////////////////////////////////////////////////////
public class PjTomography_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjTomography m_pjTomography;

   protected Button verifyButton;
   protected Button newChallengeButton;   
   
   protected int idx = 0;
   protected int score = 0;
   protected Label scoreLabel;
   protected Label messageLabel;
   
   protected Label debugLabel;
   protected int number_of_trials = 0;
   protected int max_number_of_trials = 3;
   
   protected boolean correct_answer = false;
   
   protected int level = 0;
   protected int idiom = 0;
   protected int length;

   protected Choice poly_category;
   protected Choice poly_name;

   protected Panel m_aPanel; // Transparency panel
   protected Panel m_pPanel; // Perspective panel
   
   protected Panel m_tPanel;   
   
   protected String[] title_trans = {"Tomografia",
                                     "Tomography"};
   
   protected String[] verify_trans = {"Acertei?", "Check My Answer!"};   
   protected String[] new_challenge_trans = {"Clique aqui para tentar um novo desafio!", "Click here to try a new challenge!"};   
   protected String[] score_trans = {"Pontuao: ", "Score: "};
   protected String[] final_score_trans = {"Pontuao final: ", "Final score: "};   
   
   
   protected String[] success_trans = {"Parabns!",
                              "Congratulations!"};
   
   protected String[] current_trans = {"Voc est resolvendo agora o desafio ",
                                       "You're now solving the challenge "};
  
   protected String[] failure_trans = {"Ainda no! Tente outra vez!",
                                           "Not yet! Try again!"};
   
   protected String[] end_of_the_game_trans = {"Fim de jogo! Parabns!",
                                               "End of the game! Congratulations!"};
   protected String[] end_of_the_game_wrong_trans = {"Aqui est o objeto! Fim de jogo!",
                                                     "Here is the object! End of the game!"};
   
   protected String[] help_trans = {"Aqui esto alguns cortes juntos!",
                                    "Here are some slices together!"};   
   protected String[] solution_trans = {"Aqui est o objeto!",
                                        "Here is the object!"};      

   protected String[] vis_transparency_trans = {"Transparncia:",
                                                "Transparency:"};
   protected String[] vis_perspective_trans = {"Perspectividade:",
                                               "Perspectivity:"};
   
   protected String[] vis_vertices_trans = {"Exibir vrtices",
                                            "Show vertices"};
   protected String[] vis_edges_trans = {"Exibir arestas", "Show edges"};
   protected String[] vis_elements_trans = {"Exibir faces", "Show faces"};

   protected String[] vis_panel_trans = {"Visualizao", "Visualize"};
   
   protected String[] cut_translation_trans = {"Altura do plano:", "Height:"};   
   
 
   protected String[] polyhedra_trans =
                 {"Poliedros", "Polyhedra"};
   protected String[] nonpolyhedral_surfaces_trans =
                 {"Superfcies No Polidricas", "Nonpolyhedral Surfaces"};      
   protected String[] computer_graphics_models_trans =
                 {"Modelos 3D de Computao Grfica", "Computer Graphics 3D Models"};   
   

   protected String[] of_trans = {"de", "of"};   

   protected Category polyhedra;
   protected Category curvilinear_shapes;   
   protected Category miscellany;
   
   protected Model model;
   
   protected int number_of_objects = 20;
   protected GameObject[] object =  new GameObject[number_of_objects];
   protected int[] shuffle;


   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjTomography_IP()
   {
      super();

      if (getClass() == PjTomography_IP.class)
      {
         init();
      };
   }; // End of PjTomography_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      // addTitle(title_trans[idiom]);

      add(new Label(" "));

      poly_category = new Choice();
      add(poly_category);
      poly_category.addItemListener(this);

      poly_name = new Choice();
      add(poly_name);
      poly_name.addItemListener(this);

      add(new Label(" "));
      
      verifyButton = new Button(verify_trans[idiom]);
      verifyButton.addActionListener(this);
      
      this.add(verifyButton);
      
      add(new Label(" "));
      
      scoreLabel = new Label(score_trans[idiom] + score, Label.CENTER);
      this.add(scoreLabel);
      
      add(new Label(" "));
      messageLabel = new Label(" ", Label.CENTER);
      this.add(messageLabel);
      
      this.add(new Label(" "));

      newChallengeButton = new Button(new_challenge_trans[idiom]);
      newChallengeButton.addActionListener(this);
      newChallengeButton.setVisible(false);
      newChallengeButton.setEnabled(false);
      
      this.add(newChallengeButton);
      
      this.add(new Label(" "));      
      
      this.addLine(1);        
      
      debugLabel = new Label(" ", Label.CENTER);         
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjTomography = (PjTomography) parent;
      idiom = m_pjTomography.idiom;

      // Category and name of the solid
      poly_category.add("?");
      poly_category.add(polyhedra_trans[idiom]);
      poly_category.add(nonpolyhedral_surfaces_trans[idiom]);
      poly_category.add(computer_graphics_models_trans[idiom]);
      
      
      poly_category.select("?");
      buildDataBase();
      buildChoice("?");
      poly_name.select("?");
      
      // Idiom
      verifyButton.setLabel(verify_trans[idiom]);
      newChallengeButton.setLabel(new_challenge_trans[idiom]);
      scoreLabel.setText(score_trans[idiom] + score);
      
      //////////////////////////////////////////////////////////////////
      // Add new game objects here
      //////////////////////////////////////////////////////////////////
      int i = 0;
      int easy_start;
      int easy_end;
      int medium_start;
      int medium_end;
      int hard_start;
      int hard_end;
      
      
      ////////////
      // Easy
      ////////////
      easy_start = i;
      object[i] = new GameObject(polyhedra_trans,
              new String[]
              {"Cubo",
               "Cube"},
              "off",
              "platonic-cube");
      i++;      
      
      object[i] = new GameObject(polyhedra_trans,
                                new String[]
                                {"Tetraedro Regular",
                                 "Regular Tetrahedron"},
                                "off",
                                "platonic-tetrahedron",
                                45.0, 35.3, 60.0);
      i++;
      
      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Octaedro Regular",
                                  "Regular Octahedron"},
                                 "off",
                                 "platonic-octahedron",
                                 0.0, 0.0, 45.0);
      i++;      
      
      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
                                 new String[]
                                 {"Cilindro Reto",
                                 "Right Cylinder"},
                                "off",
                                "surfaces-cylinder");
      object[i].setShowEdges(false);
      i++;                  

      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Prisma Regular Pentagonal",
                                  "Regular Pentagonal Prism"},
                                 "off",
                                 "prisms-regular-pentagonal",
                                 90.0, 0.0, 0.0);
      i++;      

      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
                                 new String[]
                                 {"Esfera",
                                  "Sphere"},
                                 "off",
                                 "surfaces-sphere");
      object[i].setShowEdges(false);
      i++;      

      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Pirmide de Base Quadrangular",
                                  "Pyramid with Quadrangular Base"},
                                 "off",
                                 "pyramids-regular-quadrangular",
                                 90.0, 0.0, 0.0);
      i++;
      
      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cone Reto",
               "Right Cone"},
              "off",
              "surfaces-cone");
      object[i].setShowEdges(false);
      i++;
      easy_end = i - 1;
      
            
      ////////////
      // Skull
      ////////////      
      object[i] = new GameObject(computer_graphics_models_trans,
              new String[]
              {"Crnio",
               "Skull"},
              "off",
              "miscellany-skull",
              0.0, 0.0, 0.0);
      object[i].setShowEdges(false);
      i++;      
      
      ////////////
      // Torus
      ////////////            
      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
                                 new String[]
                                 {"Toro",
                                  "Torus"},
                                 "off",
                                 "surfaces-torus");
      object[i].setShowEdges(false);
      i++;
  
      
      ////////////
      // Animal
      ////////////            
      object[i] = new GameObject(computer_graphics_models_trans,
              new String[]
              {"Polvo",
               "Octopus"},
              "off",
              "miscellany-octopus");
	  object[i].setShowEdges(false);
	  i++;
		
	  object[i] = new GameObject(computer_graphics_models_trans,
		              new String[]
		              {"Dromedrio",
		               "Dromedary"},
		              "off",
		              "miscellany-dromedary",
		              0.0, 0.0, 0.0);
	  object[i].setShowEdges(false);                           
	  i++;	 	  

      object[i] = new GameObject(computer_graphics_models_trans,
              new String[]
              {"Vaca",
               "Cow"},
              "off",
              "miscellany-cow",
              0.0, 0.0, 0.0);
      object[i].setShowEdges(false);
      i++;
      
      
      medium_start = i;
      
      ////////////
      // Medium
      ////////////      
      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cilindro Reto",
               "Right Cylinder"},
              "off",
              "surfaces-cylinder",
              90.0, 0.0, 0.0);
      object[i].setShowEdges(false);
      i++;
      
      
      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Tetraedro Regular",
                                  "Regular Tetrahedron"},
                                 "off",
                                 "platonic-tetrahedron");
      i++;

      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Cubo",
                                  "Cube"},
                                 "off",
                                 "platonic-cube",
                                 45.0, 0.0, 0.0);
      i++;      

      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Pirmide de Base Quadrangular",
                                  "Pyramid with Quadrangular Base"},
                                 "off",
                                 "pyramids-regular-quadrangular",
                                 0.0, 0.0, 0.0);
      i++;
      
      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cone Reto",
               "Right Cone"},
              "off",
              "surfaces-cone",
              90.0, 0.0, 0.0);
      object[i].setShowEdges(false);
      i++;
      medium_end = i - 1;
      
      hard_start = i;
      
      ////////////
      // Hard
      ////////////
      object[i] = new GameObject(polyhedra_trans,
                                 new String[]
                                 {"Cubo",
                                  "Cube"},
                                 "off",
                                 "platonic-cube",
                                 45.0, 35.4, 0.0);
      i++;      

      object[i] = new GameObject(nonpolyhedral_surfaces_trans,
              new String[]
              {"Toro",
               "Torus"},
              "off",
              "surfaces-torus",
              90.0, 0.0, 0.0);
      object[i].setShowEdges(false);
      i++;
      hard_end = i - 1;
      
                       
      // Setup the length of the game
      length = i;
      
      // Shuffle
      shuffle = new int[length];
      for (i = 0; i < length; i++)
      {
    	  shuffle[i] = i;
      }
      
      makeShuffle(shuffle, easy_start, easy_end);
      makeShuffle(shuffle, medium_start, medium_end);
      makeShuffle(shuffle, hard_start, hard_end);
      
      // Put a medium challenge in the beginning    	  
      int tmp = shuffle[2];
      shuffle[2] = shuffle[13];
      shuffle[13] = tmp;
    	  
      // Put a hard challenge in the beginning      
      tmp = shuffle[5];
      shuffle[5] = shuffle[18];
      shuffle[18] = tmp;      

      // First challenge
      messageLabel.setText(current_trans[idiom] + "1 " + of_trans[idiom] + " " + Integer.toString(length) + "!");
      messageLabel.setForeground(Color.BLUE);      
      
      // Load first model
      loadModel(0);

      // Visualization panel
      PsPanel visPanel = new PsPanel();
      
      visPanel.add(new Label(" "));
            
      visPanel.addLine(1);
      
      m_aPanel = new Panel();
      m_aPanel.setLayout(new GridLayout(1, 1));
      visPanel.add(m_aPanel);
      m_aPanel.removeAll();
      m_pjTomography.transparency.setName(vis_transparency_trans[idiom]);
      m_aPanel.add(m_pjTomography.transparency.newInspector(PsPanel.INFO_EXT));

      m_pPanel = new Panel();
      m_pPanel.setLayout(new GridLayout(1, 1));
      visPanel.add(m_pPanel);
      m_pPanel.removeAll();
      m_pjTomography.perspective.setName(vis_perspective_trans[idiom]);
      m_pPanel.add(m_pjTomography.perspective.newInspector(PsPanel.INFO_EXT));
      
      visPanel.addLine(1);

      // tabPanel.addPanel(vis_panel_trans[idiom], visPanel);

      // Translation panel
      m_tPanel = new Panel();
      m_tPanel.setLayout(new GridLayout(1, 1));
      m_tPanel.removeAll();
      m_pjTomography.translation_factor.setName(cut_translation_trans[idiom]);
      m_tPanel.add(m_pjTomography.translation_factor.newInspector(PsPanel.INFO_EXT));      
      add(m_tPanel);      
      
      
      // tabPanel.addPanel(pro_panel_trans[idiom], proPanel);
      
      // Validate
      // tabPanel.validate();

      super.setParent(parent);
   }; // End of setParent()
   

   //////////////////////////////////////////////////////////////////////////
   // Load model
   //////////////////////////////////////////////////////////////////////////
   public void makeShuffle(int[] v, int start, int end)
   {
	   for (int i = 0; i < end - start + 1; i++) 
	   {
	      int r = start + i + (int) (Math.random() * (end - start + 1 - i));
	      int swap = shuffle[start + i];	      
	      shuffle[start + i] = shuffle[r];
	      shuffle[r] = swap;           
	   } 	   
   }

   //////////////////////////////////////////////////////////////////////////
   // Load model
   //////////////////////////////////////////////////////////////////////////
   public void loadModel(int i)
   {
      m_pjTomography.loadGeometryFromFile(object[shuffle[i]]);      
      m_pjTomography.updateCutPlane();
   }

   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (m_pjTomography == event)
      {
         // Fill here!

         return(true);
      };

      return(super.update(event));
   }; // End of update()
   
   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjTomography == null)
      {
          return;
      };
         
      Object source = event.getSource();

      if (source == verifyButton)
      {    	  
    	 String user_poly_category = poly_category.getSelectedItem();
    	 String user_poly_name = poly_name.getSelectedItem();
    	 
    	 correct_answer = false;
    	 
    	 if ((user_poly_category.equals(object[shuffle[idx]].getCategory(idiom))) &&
    		 (user_poly_name.equals(object[shuffle[idx]].getName(idiom))))
    	 {
            correct_answer = true;    			 
    	 };

/*    	 
    	 if (correct_answer == false)
    	 {
    		 System.out.println("(" + user_poly_category + ")(" + object[shuffle[idx]].getCategory(idiom) + ")");
    		 System.out.println("(" + user_poly_name + ")(" + object[shuffle[idx]].getName(idiom) + ")");
    	 }
 */    	 
    	 
    	 if (correct_answer == true)
    	 {
    		 m_pjTomography.rightAnswerSoundData.play();
    		 
    		 if (m_pjTomography.showHelp == true)
    		 {
    			 m_pjTomography.removeHelpSlices();
    		 };
    		 m_pjTomography.addGeometry(m_pjTomography.poly);
    		 m_pjTomography.showSolution = true;
    		 
             if (number_of_trials < max_number_of_trials)
             {
                score = score + 3;
             };             
             
             if (idx == length - 1)
             {
                 messageLabel.setText(end_of_the_game_trans[idiom]);
                 messageLabel.setForeground(Color.BLUE);
                 verifyButton.setEnabled(false);
                 verifyButton.setVisible(false);
                 scoreLabel.setText(final_score_trans[idiom] + score);
                 scoreLabel.setForeground(Color.BLUE);
            	 poly_category.setEnabled(false);
            	 poly_name.setEnabled(false);                 
             }
             else
             {
                 debugLabel.setText(" ");
                 
                 messageLabel.setText(success_trans[idiom]);
                 messageLabel.setForeground(Color.BLUE);
                                    
                 number_of_trials = 0;
                 
            	 verifyButton.setEnabled(false);
            	 newChallengeButton.setEnabled(true);            	 
            	 newChallengeButton.setVisible(true);  
            	 poly_category.setEnabled(false);
            	 poly_name.setEnabled(false);
             }
             
        	 if (idx < length - 1)
        	 {
        	    scoreLabel.setText(score_trans[idiom] + score);
        	 }             
    	 }
    	 else
    	 {
             m_pjTomography.wrongAnswerSoundData.play();             
             
             if (number_of_trials == 0)
             {
                 messageLabel.setText(failure_trans[idiom]);            	 
             };
                          
             if (number_of_trials == 1)
             {
            	 score = score - 1;
            	 messageLabel.setText(help_trans[idiom]); 
            	 m_pjTomography.addHelpSlices();
             };
             
             if (number_of_trials == 2)
             {
            	 score = score - 2;
            	 m_pjTomography.removeHelpSlices();
            	 messageLabel.setText(solution_trans[idiom]);
            	 m_pjTomography.addGeometry(m_pjTomography.poly);
            	 m_pjTomography.showSolution = true;
            	 verifyButton.setEnabled(false);
            	 
            	 if (idx < length - 1)
            	 {
            	    newChallengeButton.setEnabled(true);            	 
            	    newChallengeButton.setVisible(true);
               	    poly_category.setEnabled(false);
            	    poly_name.setEnabled(false);            	    
            	 }
            	 else
            	 {
            		 verifyButton.setEnabled(false);
            		 verifyButton.setVisible(false);
            		 messageLabel.setText(end_of_the_game_wrong_trans[idiom]);
                     scoreLabel.setText(final_score_trans[idiom] + score);
                     scoreLabel.setForeground(Color.BLUE);            		 
            	 };
            	 
            	 poly_category.select(object[shuffle[idx]].getCategory(idiom));
            	 buildChoice(object[shuffle[idx]].getCategory(idiom));
            	 poly_name.select(object[shuffle[idx]].getName(idiom));  
            	 poly_category.setEnabled(false);
            	 poly_name.setEnabled(false);            	                           	 
             };
                                       
             messageLabel.setForeground(Color.RED);
             number_of_trials++;    		     		 
    	 }

    	 if (idx < length - 1)
    	 {
    	    scoreLabel.setText(score_trans[idiom] + score);
    	 }    	 
      };
      
      if (source == newChallengeButton)
      {    	  
    	 scoreLabel.setText(score_trans[idiom] + score);    	 
    	 poly_category.select("?");
    	 poly_name.removeAll();
    	 poly_name.add("?");
    	 verifyButton.setEnabled(true);
    	 newChallengeButton.setEnabled(false);
    	 newChallengeButton.setVisible(false);
    	 poly_category.setEnabled(true);
    	 poly_name.setEnabled(true);    	 
    	     	 
    	 idx++;
    	 m_pjTomography.loadGeometryFromFile(object[shuffle[idx]]);
    	     	 
    	 int n = idx + 1;
         messageLabel.setText(current_trans[idiom] + n + " " + of_trans[idiom] + " " + Integer.toString(length) + "!");
         messageLabel.setForeground(Color.BLUE);    
         m_pjTomography.disp.fit();
         
         number_of_trials = 0;
      };      
      
   }; // End of actionPerformed()

   
   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjTomography == null)
       {
          return;
       };

       Object source = event.getSource();

       //////////////////////////////////////////////////////////////////////
       // Names
       //////////////////////////////////////////////////////////////////////
       if (source == poly_category)
       {        
    	   	buildChoice(poly_category.getSelectedItem());    	
       }
    }; // End of itemStateChanged()



    //////////////////////////////////////////////////////////////////////////
    // Build choice of models within a given category
    //////////////////////////////////////////////////////////////////////////
    protected void buildChoice(String c)
    {
        poly_name.removeAll();
        poly_name.add("?");
        if (c.equals(polyhedra_trans[idiom]))
        {
            for (int i = 0; i < polyhedra.getNumberOfModels(); i++)
            {
               poly_name.add(polyhedra.getModel(i).getName(idiom));
            }
        }

        if (c.equals(nonpolyhedral_surfaces_trans[idiom]))
        {
                for (int i = 0; i < curvilinear_shapes.getNumberOfModels(); i++)
                {
                   poly_name.add(curvilinear_shapes.getModel(i).getName(idiom));
                }
        }
        
        
        if (c.equals(computer_graphics_models_trans[idiom]))
        {
            for (int i = 0; i < miscellany.getNumberOfModels(); i++)
            {
               poly_name.add(miscellany.getModel(i).getName(idiom));
            }
        }                
    }; // End of buildChoice()


    //////////////////////////////////////////////////////////////////////////
    // Build database of solids
    //////////////////////////////////////////////////////////////////////////
    protected void buildDataBase()
    {
        int i;

        // POLYHEDRA
        i = 0;
        polyhedra = new Category(polyhedra_trans, 8);

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Tetraedro Regular",
                 "Regular Tetrahedron"},
                "off",
                "platonic-tetrahedron",
                true, true, true,
                true,
                true));
        i++;

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Cubo",
                 "Cube"},
                "off",
                "platonic-cube",
                true, true, true,
                true,
                true));
        i++;

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Octaedro Regular",
                 "Regular Octahedron"},
                "off",
                "platonic-octahedron",
                true, true, true,
                true,
                true));
        i++;
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Dodecaedro Regular",
                 "Regular Dodecahedron"},
                "off",
                "platonic-dodecahedron",
                false, true, true,
                true,
                true));
        i++;
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Icosaedro Regular",
                 "Regular Icosahedron"},
                "off",
                "platonic-icosahedron",
                false, true, true,
                true,
                true));
        i++;                        
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Pirmide de Base Quadrangular",
                 "Pyramid with Quadrangular Base"},
                "off",
                "pyramids-regular-quadrangular",
                true, true, true,
                true,
                true));
        i++;        
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Pirmide de Base Pentagonal",
                 "Pyramid with Pentagonal Base"},
                "off",
                "pyramids-regular-pentagonal",
                false, true, true,
                true,
                true));
        i++;                
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Prisma Regular Pentagonal",
                 "Regular Pentagonal Prism"},
                "off",
                "prisms-regular-pentagonal",
                true, true, true,
                true,
                true));
        i++;

       // CURVILINEAR SHAPES
       i = 0;
       curvilinear_shapes = new Category(nonpolyhedral_surfaces_trans, 4);

       curvilinear_shapes.addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cilindro Reto",
               "Right Cylinder"},
              "off",
              "surfaces-cylinder",
              false, false, true,
              true,
              true));
      i++;
      
      curvilinear_shapes.addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cone Reto",
               "Right Cone"},
              "off",
              "surfaces-cone",
              false, false, true,
              true,
              true));
      i++;
            
      curvilinear_shapes.addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Esfera",
               "Sphere"},
              "off",
              "surfaces-sphere",
              false, false, true,
              true,
              true));
      i++;
            
      curvilinear_shapes.addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Toro",
               "Torus"},
              "off",
              "surfaces-torus",
              false, false, true,
              true,
              true));
      i++;      
      
       
      // MISCELLANY 
      i = 0;
      miscellany = new Category(computer_graphics_models_trans, 4);

      miscellany.addModel(i, new Model(computer_graphics_models_trans,
              new String[]
              {"Crnio",
               "Skull"},
              "off",
              "miscellany-skull",
              false, false, true,
              true,
              true));
      miscellany.getModel(i).setImproveUnfolding(true, 2);
      i++;
            
      miscellany.addModel(i, new Model(computer_graphics_models_trans,
               new String[]
               {"Dromedrio",
                "Dromedary"},
               "off",
               "miscellany-dromedary",
               true, true, true,
               true,
               true));
      miscellany.getModel(i).setImproveUnfolding(true, 6);
      i++;

      miscellany.addModel(i, new Model(computer_graphics_models_trans,
               new String[]
               {"Polvo",
                "Octopus"},
               "off",
               "miscellany-octopus",
               true, true, true,
               true,
               true));
       miscellany.getModel(i).setImproveUnfolding(true, 2);
       i++;
       
       miscellany.addModel(i, new Model(computer_graphics_models_trans,
               new String[]
               {"Vaca",
                "Cow"},
               "off",
               "miscellany-cow",
               true, true, true,
               true,
               true));
      miscellany.getModel(i).setImproveUnfolding(true, 6);
      i++;       
       
       
    }; // End of buildDataBase()
}; // End of PjTomography_IP class
